@@ -1,4 +1,4 @@
-/* $Id: impact.c,v 1.28.2.1 2011/01/16 12:38:14 imil Exp $ */
+/* $Id: impact.c,v 1.28.2.1.2.5 2011/01/18 17:51:37 imil Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -125,7 +125,7 @@ break_depends(Impacthead *impacthead, Pkgimpact *pimpact)
 	Pkgimpact	*rmimpact;
 	Deptreehead	rdphead, fdphead;
 	Pkgdeptree	*rdp, *fdp;
-	char		*pkgname;
+	char		*pkgname, *rpkg;
 	int			dep_break, exists;
 
 	SLIST_INIT(&rdphead);
@@ -143,10 +143,31 @@ break_depends(Impacthead *impacthead, Pkgimpact *pimpact)
 
 		SLIST_INIT(&fdphead);
 
-		/* fetch dependencies for rdp */
-		full_dep_tree(rdp->depname, DIRECT_DEPS, &fdphead);
+		/* reverse dependency is a full package name, use it and strip it */
+		XSTRDUP(rpkg, rdp->depname);
+		trunc_str(rpkg, '-', STR_BACKWARD);
 
+		/* fetch dependencies for rdp */
+		full_dep_tree(rpkg, DIRECT_DEPS, &fdphead);
+
+		/* initialize to broken dependency */
 		dep_break = 1;
+
+		/* empty full dep tree, this can't happen in normal situation.
+		 * If it does, that means that the reverse dependency we're analyzing
+		 * has no direct dependency.
+		 * Such a situation could occur if the reverse dependency is not on
+		 * the repository anymore, leading to no information regarding this
+		 * package.
+		 * So we will check if local package dependencies are satisfied by
+		 * our newly upgraded packages.
+		 */
+		if (SLIST_EMPTY(&fdphead)) {
+			free_deptree(&fdphead);
+			full_dep_tree(rpkg, LOCAL_DIRECT_DEPS, &fdphead);
+		}
+		XFREE(rpkg);
+
 		/*
 		 * browse dependencies for rdp and see if
 		 * new package to be installed matches
